@@ -9,6 +9,8 @@
 
 #include "i18n.h"
 #include "ifavourites.h"
+#include "ideclmanager.h"
+#include "ui/iuserinterface.h"
 #include "ui/imainframe.h"
 #include "gamelib.h"
 
@@ -16,6 +18,7 @@
 #include <wx/panel.h>
 #include <wx/splitter.h>
 #include "wxutil/Bitmap.h"
+#include "wxutil/Icon.h"
 
 #include "string/string.h"
 #include "eclass.h"
@@ -65,13 +68,13 @@ class EntityClassTreePopulator:
     TreeModel::Ptr _store;
 
     // Column definition
-    const ResourceTreeView::Columns& _columns;
+    const DeclarationTreeView::Columns& _columns;
 
     // Key that specifies the display folder
     std::string _folderKey;
 
-    wxIcon _folderIcon;
-    wxIcon _entityIcon;
+    wxutil::Icon _folderIcon;
+    wxutil::Icon _entityIcon;
 
     std::set<std::string> _favourites;
 
@@ -79,17 +82,16 @@ public:
 
     // Constructor
     EntityClassTreePopulator(const TreeModel::Ptr& store,
-                             const ResourceTreeView::Columns& columns)
+                             const DeclarationTreeView::Columns& columns)
     : VFSTreePopulator(store),
       _store(store),
       _columns(columns),
-      _folderKey(game::current::getValue<std::string>(FOLDER_KEY_PATH))
+      _folderKey(game::current::getValue<std::string>(FOLDER_KEY_PATH)),
+      _folderIcon(GetLocalBitmap(FOLDER_ICON)),
+      _entityIcon(GetLocalBitmap(ENTITY_ICON))
     {
-        _folderIcon.CopyFromBitmap(wxutil::GetLocalBitmap(FOLDER_ICON));
-        _entityIcon.CopyFromBitmap(wxutil::GetLocalBitmap(ENTITY_ICON));
-
         // Get the list of favourite eclasses
-        _favourites = GlobalFavouritesManager().getFavourites(decl::Type::EntityDef);
+        _favourites = GlobalFavouritesManager().getFavourites(decl::getTypeName(decl::Type::EntityDef));
     }
 
     // EntityClassVisitor implementation
@@ -108,7 +110,7 @@ public:
         // Create the folder to put this EntityClass in, depending on the value
         // of the DISPLAY_FOLDER_KEY.
         addPath(
-            eclass->getModName() + folderPath + "/" + eclass->getName(),
+            eclass->getModName() + folderPath + "/" + eclass->getDeclName(),
             [&](TreeModel::Row& row, const std::string& path,
                 const std::string& leafName, bool isFolder)
             {
@@ -121,6 +123,7 @@ public:
                 );
                 row[_columns.fullName] = leafName;
                 row[_columns.leafName] = leafName;
+                row[_columns.declName] = eclass->getDeclName();
 
                 row[_columns.isFolder] = isFolder;
                 row[_columns.isFavourite] = isFavourite;
@@ -137,10 +140,10 @@ class ThreadedEntityClassLoader final :
 {
 private:
     // Column specification struct
-    const ResourceTreeView::Columns& _columns;
+    const DeclarationTreeView::Columns& _columns;
 
 public:
-    ThreadedEntityClassLoader(const ResourceTreeView::Columns& cols) :
+    ThreadedEntityClassLoader(const DeclarationTreeView::Columns& cols) :
         ThreadedResourceTreePopulator(cols),
         _columns(cols)
     {}
@@ -202,8 +205,8 @@ EntityClassChooser::EntityClassChooser(Purpose purpose) :
 
     // Listen for defs-reloaded signal (cannot bind directly to
     // ThreadedEntityClassLoader method because it is not sigc::trackable)
-    _defsReloaded = GlobalEntityClassManager().defsReloadedSignal().connect(
-        sigc::mem_fun(this, &EntityClassChooser::loadEntityClasses)
+    _defsReloaded = GlobalDeclarationManager().signal_DeclsReloaded(decl::Type::EntityDef).connect(
+        [this]() { GlobalUserInterface().dispatch([this]() { loadEntityClasses(); }); }
     );
 
     // Setup the tree view and invoke threaded loader to get the entity classes
@@ -238,7 +241,6 @@ EntityClassChooser::~EntityClassChooser()
     _defsReloaded.disconnect();
 }
 
-// Display the singleton instance
 std::string EntityClassChooser::ChooseEntityClass(Purpose purpose, const std::string& eclassToSelect)
 {
     EntityClassChooser instance{ purpose };
@@ -317,10 +319,9 @@ void EntityClassChooser::setupTreeView()
 {
     wxPanel* parent = findNamedObject<wxPanel>(this, "EntityClassChooserLeftPane");
 
-    _treeView = new ResourceTreeView(parent, _columns, wxDV_NO_HEADER);
+    _treeView = new DeclarationTreeView(parent, decl::Type::EntityDef, _columns, wxDV_NO_HEADER);
     _treeView->AddSearchColumn(_columns.iconAndName);
     _treeView->SetExpandTopLevelItemsAfterPopulation(true);
-    _treeView->EnableFavouriteManagement(decl::Type::EntityDef);
 
     _treeView->Bind(wxEVT_DATAVIEW_SELECTION_CHANGED, &EntityClassChooser::onSelectionChanged, this);
 
@@ -356,7 +357,7 @@ void EntityClassChooser::updateUsageInfo(const std::string& eclass)
 
     // Set the usage panel to the IEntityClass' usage information string
     auto* usageText = findNamedObject<wxTextCtrl>(this, "EntityClassChooserUsageText");
-    usageText->SetValue(e ? eclass::getUsage(*e) : "");
+    usageText->SetValue(e ? eclass::getUsage(e) : "");
 }
 
 void EntityClassChooser::updateSelection()
@@ -388,7 +389,7 @@ void EntityClassChooser::updateSelection()
             {
                 _modelPreview->setModel(eclass->getAttributeValue("model"));
                 _modelPreview->setSkin(eclass->getAttributeValue("skin"));
-                defFileName->SetLabel(eclass->getDefFileName());
+                defFileName->SetLabel(eclass->getDeclFilePath());
                 return; // success
             }
         }
