@@ -445,6 +445,10 @@ public:
 	// Pass a boolean-valued "is-a-folder" column to indicate which items are actual folders.
 	virtual void SortModelFoldersFirst(const Column& stringColumn, const Column& isFolderColumn);
 
+    // Sort the model by a string-valued column, sorting folders on top, starting at the given item.
+    // See also the other SortModelFoldersFirst overload.
+    virtual void SortModelFoldersFirst(const wxDataViewItem& startItem, const Column& stringColumn, const Column& isFolderColumn);
+
     using FolderCompareFunction = std::function<int(const wxDataViewItem&, const wxDataViewItem&)>;
 
     // Sort the model by a string-valued column, sorting folders on top.
@@ -454,6 +458,11 @@ public:
 	virtual void SortModelFoldersFirst(const Column& stringColumn, const Column& isFolderColumn, 
         const FolderCompareFunction& customFolderSortFunc);
 
+    // Sort the model by a string-valued column, sorting folders on top, starting at the given item.
+    // See also the other SortModelFoldersFirst overload.
+    virtual void SortModelFoldersFirst(const wxDataViewItem& startItem, const Column& stringColumn, 
+        const Column& isFolderColumn, const FolderCompareFunction& customFolderSortFunc);
+
     // Find the given string needle in the given column (searches the entire tree)
 	virtual wxDataViewItem FindString(const std::string& needle, const Column& column);
 
@@ -466,6 +475,12 @@ public:
     // Find the given number needle in the given column (searches only the subtree given by the startNode item)
 	virtual wxDataViewItem FindInteger(long needle, const Column& column, const wxDataViewItem& startNode);
 
+    // Find the item matching the predicate (searches the entire tree)
+    virtual wxDataViewItem FindItem(const std::function<bool(const TreeModel::Row&)>& predicate);
+
+    // Find the item matching the predicate (searches from the given item)
+    virtual wxDataViewItem FindItem(const std::function<bool(const TreeModel::Row&)>& predicate, const wxDataViewItem& startNode);
+
     // Returns true if any of the given columns in the given row contains the string value
     // Set lowerStrings to true to convert the column values to lowercase first (the value is not touched
     // and needs to be made lowercase by the calling code).
@@ -516,6 +531,10 @@ public:
 	virtual int Compare(const wxDataViewItem& item1, const wxDataViewItem& item2,
                         unsigned int column, bool ascending) const override;
 
+    // Forces client data views to refresh this subtree by sending add/remove events
+    // for each of the children of the given item
+    virtual void SendSubtreeRefreshEvents(wxDataViewItem& parentItem);
+
 protected:
 	// Returns a reference to the actual rootnode, only allowed for use in subclasses
 	virtual const NodePtr& getRootNode() const;
@@ -523,7 +542,7 @@ protected:
 protected:
 	void ForeachNodeRecursive(const TreeModel::NodePtr& node, const VisitFunction& visitFunction);
 	void ForeachNodeRecursiveReverse(const TreeModel::NodePtr& node, const TreeModel::VisitFunction& visitFunction);
-	void SortModelRecursive(const TreeModel::NodePtr& node, const TreeModel::SortFunction& sortFunction);
+	void SortModelRecursively(Node* node, const TreeModel::SortFunction& sortFunction);
 
 	// Sort functor for the SortModelFoldersFirst() method, uses the stringCompare method to compare the actual text values
     // Pass CompareStringVariants or CompareIconTextVariants as stringCompare.
