@@ -46,6 +46,7 @@ RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
 	_glWidget(new wxutil::GLWidget(_mainPanel, std::bind(&RenderPreview::drawPreview, this), "RenderPreview")),
     _initialised(false),
 	_renderGrid(registry::getValue<bool>(RKEY_RENDERPREVIEW_SHOWGRID)),
+    _enableLightingModeAtStart(false),
     _renderSystem(GlobalRenderSystemFactory().createRenderSystem()),
     _viewOrigin(0, 0, 0),
     _viewAngles(0, 0, 0),
@@ -81,6 +82,11 @@ RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
         std::bind(&RenderPreview::onGLMouseRelease, this, std::placeholders::_1));
 }
 
+void RenderPreview::setStartupLightingMode(bool enableAtStart)
+{
+    _enableLightingModeAtStart = enableAtStart;
+}
+
 void RenderPreview::setupToolbars(bool enableAnimation)
 {
     wxToolBar* toolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewAnimToolbar");
@@ -219,7 +225,7 @@ void RenderPreview::initialisePreview()
 
     if (_renderSystem->shaderProgramsAvailable())
     {
-        setLightingModeEnabled(false);
+        setLightingModeEnabled(_enableLightingModeAtStart);
     }
 
     // In case it didn't happen until now, make sure the rendersystem is realised
@@ -228,6 +234,11 @@ void RenderPreview::initialisePreview()
     updateModelViewMatrix();
 }
 
+const Vector3& RenderPreview::getViewOrigin()
+{
+    return _viewOrigin;
+}
+
 void RenderPreview::setViewOrigin(const Vector3& origin)
 {
     _viewOrigin = origin;
@@ -235,6 +246,11 @@ void RenderPreview::setViewOrigin(const Vector3& origin)
     updateModelViewMatrix();
 }
 
+const Vector3& RenderPreview::getViewAngles()
+{
+    return _viewAngles;
+}
+
 void RenderPreview::setViewAngles(const Vector3& angles)
 {
     _viewAngles = angles;
