@@ -33,7 +33,6 @@
 #include "CameraSettings.h"
 #include <functional>
 #include <sigc++/retype_return.h>
-#include "render/VectorLightList.h"
 #include "render/CamRenderer.h"
 
 namespace ui
@@ -127,6 +126,13 @@ CamWnd::CamWnd(wxWindow* parent) :
         radiant::IMessage::Type::TextureChanged,
         radiant::TypeListener<radiant::TextureChangedMessage>(
             sigc::mem_fun(this, &CamWnd::handleTextureChanged)));
+
+    _renderer.reset(new render::CamRenderer(_view, _shaders));
+
+    // Refresh the camera view when shadows are enabled/disabled
+    GlobalRegistry().signalForKey(RKEY_ENABLE_SHADOW_MAPPING).connect(
+        sigc::mem_fun(this, &CamWnd::queueDraw)
+    );
 }
 
 wxWindow* CamWnd::getMainWidget() const
@@ -143,14 +149,19 @@ void CamWnd::constructToolbar()
     const wxToolBarToolBase* flatShadeBtn = getToolBarToolByLabel(_camToolbar, "flatShadeBtn");
     const wxToolBarToolBase* texturedBtn = getToolBarToolByLabel(_camToolbar, "texturedBtn");
     const wxToolBarToolBase* lightingBtn = getToolBarToolByLabel(_camToolbar, "lightingBtn");
+    const wxToolBarToolBase* shadowLightingBtn = getToolBarToolByLabel(_camToolbar, "shadowBtn");
     const wxToolBarToolBase* gridButton = getToolBarToolByLabel(_camToolbar, "drawGridButton");
 
     if (!GlobalRenderSystem().shaderProgramsAvailable())
     {
         //lightingBtn->set_sensitive(false);
         _camToolbar->EnableTool(lightingBtn->GetId(), false);
+        _camToolbar->EnableTool(shadowLightingBtn->GetId(), false);
     }
 
+    auto toggleShadowMappingEvent = GlobalEventManager().findEvent("ToggleShadowMapping");
+    GlobalEventManager().registerToolItem("ToggleShadowMapping", shadowLightingBtn);
+
     // Listen for render-mode changes, and set the correct active button to
     // start with.
     getCameraSettings()->signalRenderModeChanged().connect(
@@ -216,8 +227,10 @@ void CamWnd::onGLExtensionsInitialised()
 {
     // If lighting is not available, grey out the lighting button
     const wxToolBarToolBase* lightingBtn = getToolBarToolByLabel(_camToolbar, "lightingBtn");
+    const wxToolBarToolBase* shadowBtn = getToolBarToolByLabel(_camToolbar, "shadowBtn");
 
     _camToolbar->EnableTool(lightingBtn->GetId(), GlobalRenderSystem().shaderProgramsAvailable());
+    _camToolbar->EnableTool(shadowBtn->GetId(), GlobalRenderSystem().shaderProgramsAvailable());
 }
 
 void CamWnd::setFarClipButtonSensitivity()
@@ -782,29 +795,39 @@ void CamWnd::Cam_Draw()
                             | RENDER_POLYGONSTIPPLE;
     }
 
+    IRenderResult::Ptr result;
+
     // Main scene render
     {
+        GlobalRenderSystem().startFrame();
+
+        _renderer->prepare();
+
         // Front end (renderable collection from scene)
-        render::CamRenderer renderer(_view, _shaders);
-        render::RenderableCollectionWalker::CollectRenderablesInScene(renderer, _view);
+        render::RenderableCollectionWalker::CollectRenderablesInScene(*_renderer, _view);
 
         // Accumulate render statistics
-        _renderStats.setLightCount(renderer.getVisibleLights(),
-                                   renderer.getTotalLights());
         _renderStats.frontEndComplete();
 
         // Render any active mousetools
         for (const ActiveMouseTools::value_type& i : _activeMouseTools)
         {
-            i.second->render(GlobalRenderSystem(), renderer, _view);
+            i.second->render(GlobalRenderSystem(), *_renderer, _view);
         }
 
-        // Back end (submit to shaders and do the actual render)
-        renderer.submitToShaders(
-            getCameraSettings()->getRenderMode() == RENDER_MODE_LIGHTING
-        );
-        GlobalRenderSystem().render(allowedRenderFlags, _camera->getModelView(),
-                                    _camera->getProjection(), _view.getViewer());
+        if (getCameraSettings()->getRenderMode() == RENDER_MODE_LIGHTING)
+        {
+            // Lit mode
+            result = GlobalRenderSystem().renderLitScene(allowedRenderFlags, _view);
+        }
+        else
+        {
+            result = GlobalRenderSystem().renderFullBrightScene(RenderViewType::Camera, allowedRenderFlags, _view);
+        }
+
+        _renderer->cleanup();
+
+        GlobalRenderSystem().endFrame();
     }
 
     // greebo: Draw the clipper's points (skipping the depth-test)
@@ -871,10 +894,15 @@ void CamWnd::Cam_Draw()
     // Render the stats and timing text. This may include culling stats in
     // debug builds.
     glRasterPos3f(4.0f, static_cast<float>(_camera->getDeviceHeight()) - 4.0f, 0.0f);
-    std::string statString = _view.getCullStats();
-    if (!statString.empty())
+
+    auto statString = result->toString();
+
+    if (getCameraSettings()->getRenderMode() != RENDER_MODE_LIGHTING)
+    {
         statString += " | ";
-    statString += _renderStats.getStatString();
+        statString += _renderStats.getStatString();
+    }
+
     _glFont->drawString(statString);
 
     drawTime();
@@ -964,12 +992,12 @@ camera::ICameraView& CamWnd::getCamera()
 
 void CamWnd::captureStates()
 {
-    _shaders.faceHighlightShader = GlobalRenderSystem().capture("$CAM_HIGHLIGHT");
-    _shaders.primitiveHighlightShader = GlobalRenderSystem().capture("$CAM_OVERLAY");
-    _shaders.mergeActionShaderAdd = GlobalRenderSystem().capture("$MERGE_ACTION_ADD");
-    _shaders.mergeActionShaderChange = GlobalRenderSystem().capture("$MERGE_ACTION_CHANGE");
-    _shaders.mergeActionShaderRemove = GlobalRenderSystem().capture("$MERGE_ACTION_REMOVE");
-    _shaders.mergeActionShaderConflict = GlobalRenderSystem().capture("$MERGE_ACTION_CONFLICT");
+    _shaders.faceHighlightShader = GlobalRenderSystem().capture(BuiltInShaderType::ColouredPolygonOverlay);
+    _shaders.primitiveHighlightShader = GlobalRenderSystem().capture(BuiltInShaderType::HighlightedPolygonOutline);
+    _shaders.mergeActionShaderAdd = GlobalRenderSystem().capture(BuiltInShaderType::CameraMergeActionOverlayAdd);
+    _shaders.mergeActionShaderChange = GlobalRenderSystem().capture(BuiltInShaderType::CameraMergeActionOverlayChange);
+    _shaders.mergeActionShaderRemove = GlobalRenderSystem().capture(BuiltInShaderType::CameraMergeActionOverlayRemove);
+    _shaders.mergeActionShaderConflict = GlobalRenderSystem().capture(BuiltInShaderType::CameraMergeActionOverlayConflict);
 }
 
 void CamWnd::releaseStates() 
