@@ -35,7 +35,7 @@
 #include "ui/overlay/OverlayDialog.h"
 #include "ui/prefdialog/PrefDialog.h"
 #include "ui/Documentation.h"
-#include "log/Console.h"
+#include "ui/console/Console.h"
 #include "ui/lightinspector/LightInspector.h"
 #include "ui/patch/PatchInspector.h"
 #include "ui/surfaceinspector/SurfaceInspector.h"
@@ -105,7 +105,10 @@ const StringSet& UserInterfaceModule::getDependencies() const
         MODULE_MRU_MANAGER,
         MODULE_MAINFRAME,
         MODULE_MOUSETOOLMANAGER,
-        MODULE_MAP
+        MODULE_MAP,
+        MODULE_PATCH,
+        MODULE_BRUSHCREATOR,
+        MODULE_TEXTOOL_SELECTIONSYSTEM,
     };
 
 	return _dependencies;
@@ -113,8 +116,6 @@ const StringSet& UserInterfaceModule::getDependencies() const
 
 void UserInterfaceModule::initialiseModule(const IApplicationContext& ctx)
 {
-	rMessage() << getName() << "::initialiseModule called." << std::endl;
-
 	// Output the wxWidgets version to the logfile
 	std::string wxVersion = string::to_string(wxMAJOR_VERSION) + ".";
 	wxVersion += string::to_string(wxMINOR_VERSION) + ".";
@@ -237,6 +238,9 @@ void UserInterfaceModule::initialiseModule(const IApplicationContext& ctx)
         GlobalMenuManager().setVisibility("main/help/userGuideLocal", false);
     });
 #endif
+
+    _reloadMaterialsConn = GlobalDeclarationManager().signal_DeclsReloaded(decl::Type::Material)
+        .connect([this]() { dispatch([]() { GlobalMainFrame().updateAllWindows(); }); });
 }
 
 void UserInterfaceModule::shutdownModule()
@@ -248,6 +252,7 @@ void UserInterfaceModule::shutdownModule()
 	GlobalRadiantCore().getMessageBus().removeListener(_execFailedListener);
 	GlobalRadiantCore().getMessageBus().removeListener(_notificationListener);
 
+    _reloadMaterialsConn.disconnect();
 	_coloursUpdatedConn.disconnect();
 	_entitySettingsConn.disconnect();
     _mapEditModeChangedConn.disconnect();
@@ -364,20 +369,6 @@ void UserInterfaceModule::applyEntityVertexColours()
 	settings.setLightVertexColour(LightEditVertexType::Selected, GlobalColourSchemeManager().getColour("light_vertex_selected"));
 }
 
-void UserInterfaceModule::refreshShadersCmd(const cmd::ArgumentList& args)
-{
-	// Disable screen updates for the scope of this function
-	auto blocker = GlobalMainFrame().getScopedScreenUpdateBlocker(_("Processing..."), _("Loading Shaders"));
-
-	// Reload the Shadersystem, this will also trigger an
-	// OpenGLRenderSystem unrealise/realise sequence as the rendersystem
-	// is attached to this class as Observer
-	// We can't do this refresh() operation in a thread it seems due to context binding
-	GlobalMaterialManager().refresh();
-
-	GlobalMainFrame().updateAllWindows();
-}
-
 void UserInterfaceModule::registerUICommands()
 {
 	TexTool::registerCommands();
@@ -416,7 +407,6 @@ void UserInterfaceModule::registerUICommands()
 
 	GlobalCommandSystem().addCommand("EntityClassTree", EClassTree::ShowDialog);
 	GlobalCommandSystem().addCommand("EntityList", EntityList::toggle);
-	GlobalCommandSystem().addCommand("RefreshShaders", std::bind(&UserInterfaceModule::refreshShadersCmd, this, std::placeholders::_1));
 
 	// ----------------------- Bind Events ---------------------------------------
 
@@ -427,15 +417,19 @@ void UserInterfaceModule::registerUICommands()
 	GlobalCommandSystem().addCommand("CreateNewLayerDialog", CreateLayerDialog::CreateNewLayer,
 		{ cmd::ARGTYPE_STRING | cmd::ARGTYPE_OPTIONAL });
 
-	GlobalCommandSystem().addCommand("BulgePatchDialog", BulgePatchDialog::BulgePatchCmd);
-	GlobalCommandSystem().addCommand("PatchCapDialog", PatchCapDialog::Show);
-	GlobalCommandSystem().addCommand("ThickenPatchDialog", PatchThickenDialog::Show);
-	GlobalCommandSystem().addCommand("CreateSimplePatchDialog", PatchCreateDialog::Show);
+    GlobalCommandSystem().addWithCheck("BulgePatchDialog", BulgePatchDialog::BulgePatchCmd,
+                                       selection::pred::havePatch);
+    GlobalCommandSystem().addWithCheck("PatchCapDialog", PatchCapDialog::Show,
+                                       selection::pred::havePatch);
+    GlobalCommandSystem().addWithCheck("ThickenPatchDialog", PatchThickenDialog::Show,
+                                       selection::pred::havePatch);
+    GlobalCommandSystem().addCommand("CreateSimplePatchDialog", PatchCreateDialog::Show);
 
-	GlobalCommandSystem().addCommand("ExportCollisionModelDialog", ExportCollisionModelDialog::Show);
-	GlobalCommandSystem().addCommand("QueryBrushPrefabSidesDialog", QuerySidesDialog::Show, { cmd::ARGTYPE_INT });
+    GlobalCommandSystem().addCommand("ExportCollisionModelDialog", ExportCollisionModelDialog::Show);
+    GlobalCommandSystem().addWithCheck("QueryBrushPrefabSidesDialog", QuerySidesDialog::Show,
+                                       selection::pred::haveBrush, {cmd::ARGTYPE_INT});
 
-	// Set up the CloneSelection command to react on key up events only
+    // Set up the CloneSelection command to react on key up events only
 	GlobalEventManager().addCommand("CloneSelection", "CloneSelection", true); // react on keyUp
 
 	GlobalEventManager().addRegistryToggle("ToggleRotationPivot", "user/ui/rotationPivotIsOrigin");
