@@ -17,9 +17,7 @@
 namespace ui
 {
 
-FloatPropertyEditor::FloatPropertyEditor(wxWindow* parent, IEntitySelection& entities,
-										 const std::string& key,
-										 const std::string& options)
+FloatPropertyEditor::FloatPropertyEditor(wxWindow* parent, IEntitySelection& entities, const ITargetKey::Ptr& key)
 : PropertyEditor(entities),
   _spinCtrl(nullptr),
   _key(key)
@@ -31,39 +29,18 @@ FloatPropertyEditor::FloatPropertyEditor(wxWindow* parent, IEntitySelection& ent
 	// Register the main widget in the base class
 	setMainWidget(mainVBox);
 
-	// Split the options string to get min and max values
-	std::vector<std::string> values;
-	string::split(values, options, ",");
-	if (values.size() != 2)
-		return;
-
-	// Attempt to cast to min and max floats
-	float min, max;
-	try
-	{
-		min = std::stof(values[0]);
-		max = std::stof(values[1]);
-	}
-	catch (std::invalid_argument&)
-	{
-		rError() << "[radiant] FloatPropertyEditor failed to parse options string "
-			<< "\"" << options << "\"" << std::endl;
-		return;
-	}
-
 	// Create the HScale and pack into widget
 	_spinCtrl = new wxSpinCtrlDouble(parent, wxID_ANY);
 
 	_spinCtrl->SetIncrement(1.0);
-	_spinCtrl->SetRange(min, max);
 	_spinCtrl->SetMinSize(wxSize(75, -1));
 
 	// Set the initial value if the entity has one
 	updateFromEntity();
 
 	// Create and pack in the Apply button
-	wxButton* applyButton = new wxButton(mainVBox, wxID_APPLY, _("Apply..."));
-	applyButton->Connect(wxEVT_BUTTON, wxCommandEventHandler(FloatPropertyEditor::_onApply), NULL, this);
+	wxButton* applyButton = new wxButton(mainVBox, wxID_APPLY, _("Apply"));
+	applyButton->Bind(wxEVT_BUTTON, &FloatPropertyEditor::_onApply, this);
 
 	mainVBox->GetSizer()->Add(_spinCtrl, 0, wxALIGN_CENTER_VERTICAL);
 	mainVBox->GetSizer()->Add(applyButton, 0, wxALIGN_CENTER_VERTICAL | wxALL, 6);
@@ -73,7 +50,7 @@ void FloatPropertyEditor::updateFromEntity()
 {
 	if (_spinCtrl == nullptr) return;
 
-	float value = string::convert<float>(_entities.getSharedKeyValue(_key, false), 0);
+	auto value = string::convert<float>(_entities.getSharedKeyValue(_key->getFullKey(), false), 0);
 	
 	_spinCtrl->SetValue(value);
 }
@@ -82,7 +59,7 @@ void FloatPropertyEditor::_onApply(wxCommandEvent& ev)
 {
 	float value = static_cast<float>(_spinCtrl->GetValue());
 
-	setKeyValue(_key, string::to_string(value));
+	setKeyValue(_key->getFullKey(), string::to_string(value));
 }
 
 }
