@@ -11,7 +11,7 @@
 #include "iregistry.h"
 #include "iradiant.h"
 
-#include "log/Console.h"
+#include "ui/console/Console.h"
 #include "xyview/GlobalXYWnd.h"
 #include "camera/CameraWndManager.h"
 
@@ -81,8 +81,6 @@ const StringSet& MainFrame::getDependencies() const
 
 void MainFrame::initialiseModule(const IApplicationContext& ctx)
 {
-	rMessage() << "MainFrame::initialiseModule called." << std::endl;
-
 	// Add another page for Multi-Monitor stuff
 	IPreferencePage& page = GlobalPreferenceSystem().getPage(_("Settings/Multi Monitor"));
 
@@ -149,14 +147,12 @@ void MainFrame::initialiseModule(const IApplicationContext& ctx)
 	);
 
     // When the eclass defs are in progress of being loaded, block all updates
-    GlobalEntityClassManager().defsLoadingSignal().connect(
+    _defsLoadingSignal = GlobalDeclarationManager().signal_DeclsReloading(decl::Type::EntityDef).connect(
         [this]() { _defLoadingBlocksUpdates = true; }
     );
 
-    GlobalEntityClassManager().defsLoadedSignal().connect([this]()
-        {
-            _defLoadingBlocksUpdates = false;
-        }
+    _defsLoadedSignal = GlobalDeclarationManager().signal_DeclsReloaded(decl::Type::EntityDef).connect(
+        [this]() { _defLoadingBlocksUpdates = false; }
     );
 
 	// Subscribe for the post-module init event
@@ -171,6 +167,9 @@ void MainFrame::shutdownModule()
 	_mapNameChangedConn.disconnect();
 	_mapModifiedChangedConn.disconnect();
 
+    _defsLoadingSignal.disconnect();
+    _defsLoadedSignal.disconnect();
+
 	disableScreenUpdates();
 }
 
