@@ -2,12 +2,14 @@
 
 #include "i18n.h"
 #include "ui/imainframe.h"
+#include "ideclmanager.h"
 #include "imap.h"
 #include "ui/igroupdialog.h"
 #include "ipreferencesystem.h"
 #include "ishaders.h"
 #include "ishaderclipboard.h"
 #include "ifavourites.h"
+#include "ui/iuserinterface.h"
 
 #include "wxutil/MultiMonitor.h"
 #include "wxutil/dataview/ResourceTreeViewToolbar.h"
@@ -30,6 +32,7 @@
 
 #include <functional>
 #include "string/predicate.h"
+#include "ui/UserInterfaceModule.h"
 
 namespace ui
 {
@@ -121,23 +124,23 @@ void MediaBrowser::onMapEvent(IMap::MapEvent ev)
 
 std::string MediaBrowser::getSelection()
 {
-    return _treeView->GetSelectedFullname();
+    return _treeView->GetSelectedDeclName();
 }
 
 // Set the selection in the treeview
 void MediaBrowser::setSelection(const std::string& selection)
 {
-    _treeView->SetSelectedFullname(selection);
+    _treeView->SetSelectedDeclName(selection);
 }
 
 void MediaBrowser::onMaterialDefsLoaded()
 {
-    _treeView->Populate();
+    GlobalUserInterface().dispatch([this]() { _treeView->Populate(); });
 }
 
 void MediaBrowser::onMaterialDefsUnloaded()
 {
-    _treeView->Clear();
+    GlobalUserInterface().dispatch([this]() { _treeView->Clear(); });
 }
 
 void MediaBrowser::_onTreeViewSelectionChanged(wxDataViewEvent& ev)
@@ -179,7 +182,8 @@ const StringSet& MediaBrowser::getDependencies() const
         MODULE_SHADERCLIPBOARD,
         MODULE_MAINFRAME,
         MODULE_FAVOURITES_MANAGER,
-        MODULE_MAP
+        MODULE_MAP,
+        MODULE_USERINTERFACE,
     };
 
 	return _dependencies;
@@ -187,8 +191,6 @@ const StringSet& MediaBrowser::getDependencies() const
 
 void MediaBrowser::initialiseModule(const IApplicationContext& ctx)
 {
-	rMessage() << getName() << "::initialiseModule called." << std::endl;
-
 	GlobalCommandSystem().addCommand("ToggleMediaBrowser", sigc::mem_fun(this, &MediaBrowser::togglePage));
 
 	// We need to create the liststore and widgets before attaching ourselves
@@ -201,15 +203,13 @@ void MediaBrowser::initialiseModule(const IApplicationContext& ctx)
 		sigc::mem_fun(*this, &MediaBrowser::onMainFrameConstructed)
 	);
 
-	// Attach to the MaterialManager to get notified on unrealise/realise
-	// events, in which case we're reloading the media tree
-	_materialDefsLoaded = GlobalMaterialManager().signal_DefsLoaded().connect(
-		sigc::mem_fun(*this, &MediaBrowser::onMaterialDefsLoaded)
-	);
+    // Attach to the DeclarationManager to get notified on unrealise/realise
+    // events, in which case we're reloading the media tree
+    _materialDefsLoaded = GlobalDeclarationManager().signal_DeclsReloaded(decl::Type::Material)
+        .connect(sigc::mem_fun(*this, &MediaBrowser::onMaterialDefsLoaded));
 
-	_materialDefsUnloaded = GlobalMaterialManager().signal_DefsUnloaded().connect(
-		sigc::mem_fun(*this, &MediaBrowser::onMaterialDefsUnloaded)
-	);
+    _materialDefsUnloaded = GlobalDeclarationManager().signal_DeclsReloading(decl::Type::Material)
+        .connect(sigc::mem_fun(*this, &MediaBrowser::onMaterialDefsUnloaded));
 
 	_shaderClipboardConn = GlobalShaderClipboard().signal_sourceChanged().connect(
 		sigc::mem_fun(this, &MediaBrowser::onShaderClipboardSourceChanged)
