@@ -29,8 +29,8 @@
 #include "wxutil/Bitmap.h"
 #include "ui/UserInterfaceModule.h"
 #include "registry/Widgets.h"
-
-#include <functional>
+#include "wxutil/menu/IconTextMenuItem.h"
+#include "wxutil/sourceview/DeclarationSourceView.h"
 
 namespace ui
 {
@@ -38,11 +38,14 @@ namespace ui
 // CONSTANTS
 namespace
 {
-    const char* const MODELSELECTOR_TITLE = N_("Choose Model");
+    constexpr const char* const MODELSELECTOR_TITLE = N_("Choose Model");
+    constexpr const char* const SHOW_ECLASS_DEFINITION_TEXT = N_("Show Definition");
+    constexpr const char* const SHOW_ECLASS_DEFINITION_ICON = "icon_classname.png";
 
     const std::string RKEY_BASE = "user/ui/modelSelector/";
     const std::string RKEY_SPLIT_POS = RKEY_BASE + "splitPos";
     const std::string RKEY_SHOW_SKINS = RKEY_BASE + "showSkinsInTree";
+    const std::string RKEY_LAST_SELECTED_MODEL = RKEY_BASE + "lastSelectedModel";
 }
 
 // Constructor.
@@ -53,8 +56,7 @@ ModelSelector::ModelSelector() :
     _treeView(nullptr),
 	_infoTable(nullptr),
     _materialsList(nullptr),
-	_lastModel(""),
-	_lastSkin(""),
+    _relatedEntityView(nullptr),
 	_showOptions(true)
 {
     // Set the default size of the window
@@ -107,25 +109,109 @@ ModelSelector::ModelSelector() :
 
 void ModelSelector::setupAdvancedPanel(wxWindow* parent)
 {
+    auto detailsPage = findNamedPanel(parent, "ModelDetailsPanel");
+
 	// Create info panel
-	_infoTable = new wxutil::KeyValueTable(parent);
+	_infoTable = new wxutil::KeyValueTable(detailsPage);
 	_infoTable->SetMinClientSize(wxSize(-1, 140));
 
-	_materialsList = new MaterialsList(parent, _modelPreview->getRenderSystem());
+    detailsPage->GetSizer()->Add(_infoTable, 1, wxEXPAND);
+
+    auto materialsPage = findNamedPanel(parent, "ModelMaterialsPanel");
+
+	_materialsList = new MaterialsList(materialsPage, _modelPreview->getRenderSystem());
     _materialsList->SetMinClientSize(wxSize(-1, 140));
 
+    materialsPage->GetSizer()->Add(_materialsList, 1, wxEXPAND);
+
 	// Refresh preview when material visibility changed
     _materialsList->signal_visibilityChanged().connect(
 		sigc::mem_fun(*_modelPreview, &wxutil::ModelPreview::queueDraw)
     );
 
-    parent->GetSizer()->Add(_materialsList, 0, wxEXPAND | wxTOP, 6);
-	parent->GetSizer()->Add(_infoTable, 0, wxEXPAND | wxTOP, 6);
+    auto entityPage = findNamedPanel(parent, "ModelRelatedEntitiesPanel");
+
+    _relatedEntityStore = new wxutil::TreeModel(_relatedEntityColumns, true);
+    _relatedEntityView = wxutil::TreeView::CreateWithModel(entityPage, _relatedEntityStore.get());
+
+    _relatedEntityView->AppendTextColumn(_("Entity Class"), _relatedEntityColumns.eclassName.getColumnIndex(),
+        wxDATAVIEW_CELL_INERT, wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT, wxDATAVIEW_COL_SORTABLE);
+    _relatedEntityView->AppendTextColumn(_("Skin"), _relatedEntityColumns.skin.getColumnIndex(),
+        wxDATAVIEW_CELL_INERT, wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT, wxDATAVIEW_COL_SORTABLE);
+
+    _relatedEntityView->Bind(wxEVT_DATAVIEW_SELECTION_CHANGED, &ModelSelector::onRelatedEntitySelectionChange, this);
+    _relatedEntityView->Bind(wxEVT_DATAVIEW_ITEM_ACTIVATED, &ModelSelector::onRelatedEntityActivated, this);
+
+    _relatedEntityView->Bind(wxEVT_DATAVIEW_ITEM_CONTEXT_MENU, [&] (auto&)
+    {
+        _relatedEntityContextMenu->show(this);
+    });
+
+    // Construct the context menu
+    _relatedEntityContextMenu = std::make_shared<wxutil::PopupMenu>();
+    _relatedEntityContextMenu->addItem(
+        new wxutil::IconTextMenuItem(_(SHOW_ECLASS_DEFINITION_TEXT), SHOW_ECLASS_DEFINITION_ICON),
+        std::bind(&ModelSelector::onShowClassDefinition, this),
+        [this]() { return _relatedEntityView->GetSelection().IsOk(); }
+    );
+
+    entityPage->GetSizer()->Prepend(_relatedEntityView, 1, wxEXPAND);
+}
+
+void ModelSelector::onShowClassDefinition()
+{
+    auto item = _relatedEntityView->GetSelection();
+    if (!item.IsOk()) return;
+
+    wxutil::TreeModel::Row row(item, *_relatedEntityStore.get());
+    std::string eclass = row[_relatedEntityColumns.eclassName];
+
+    auto view = new wxutil::DeclarationSourceView(this);
+    view->setDeclaration(decl::Type::EntityDef, eclass);
+
+    view->ShowModal();
+    view->Destroy();
+}
+
+void ModelSelector::onRelatedEntitySelectionChange(wxDataViewEvent& ev)
+{
+    // Adjust the model preview to use the same skin
+    auto item = _relatedEntityView->GetSelection();
+
+    if (!item.IsOk()) return;
+
+    wxutil::TreeModel::Row row(item, *_relatedEntityStore.get());
+
+    std::string skin = row[_relatedEntityColumns.skin];
+    _modelPreview->setSkin(skin);
+
+    // Sync the selection in the model tree to match this model/skin combination
+    _treeView->SetSelectedSkin(skin);
+}
+
+void ModelSelector::onRelatedEntityActivated(wxDataViewEvent& ev)
+{
+    auto item = _relatedEntityView->GetSelection();
+
+    if (!item.IsOk()) return;
+
+    wxutil::TreeModel::Row row(item, *_relatedEntityStore.get());
+
+    std::string eclass = row[_relatedEntityColumns.eclassName];
+
+    _result.objectKind = Result::ObjectKind::EntityClass;
+    _result.name = eclass;
+    _result.skin = "";
+    _result.createClip = false;
+
+    EndModal(wxID_OK);
+    Hide();
 }
 
 void ModelSelector::cancelDialog()
 {
 	_panedPosition.saveToPath(RKEY_SPLIT_POS);
+    _result = Result();
 
     EndModal(wxID_CANCEL);
 	Hide();
@@ -183,9 +269,8 @@ void ModelSelector::onTreeViewPopulationFinished(wxutil::ResourceTreeView::Popul
 }
 
 // Show the dialog and enter recursive main loop
-ModelSelectorResult ModelSelector::showAndBlock(const std::string& curModel,
-                                                bool showOptions,
-                                                bool showSkins)
+ModelSelector::Result ModelSelector::showAndBlock(const std::string& curModel,
+    bool showOptions, bool showSkins)
 {
     // Hide the Show Skins button if skins should not be shown for this invocation
     if (showSkins) {
@@ -197,31 +282,37 @@ ModelSelectorResult ModelSelector::showAndBlock(const std::string& curModel,
         _treeView->SetShowSkins(false);
     }
 
-    _treeView->SetSelectedFullname(curModel);
-    showInfoForSelectedModel();
+    if (!curModel.empty())
+    {
+        _treeView->SetSelectedFullname(curModel);
+    }
+    else
+    {
+        // If no model provided, use the selection from the previous session
+        _treeView->SetSelectedFullname(registry::getValue<std::string>(RKEY_LAST_SELECTED_MODEL));
+    }
 
     _showOptions = showOptions;
 
     // Conditionally hide the options
     findNamedObject<wxPanel>(this, "ModelSelectorOptionsPanel")->Show(_showOptions);
 
+    _result = Result();
+
     // show and enter recursive main loop.
     int returnCode = ShowModal();
 
     // Remove the model from the preview's scenegraph before returning
     _modelPreview->setModel("");
+    _infoModel.clear();
+    _infoSkin.clear();
 
-    // Return selected model/skin, or an empty result if the dialog was cancelled
-    if (returnCode == wxID_OK)
-        return {_lastModel, _lastSkin,
-                findNamedObject<wxCheckBox>(this, "ModelSelectorMonsterClipOption")->GetValue()};
-    else
-        return {};
+    return returnCode == wxID_OK ? _result : Result();
 }
 
 // Static function to display the instance, and return the selected model to the
 // calling function
-ModelSelectorResult ModelSelector::chooseModel(const std::string& curModel,
+ModelSelector::Result ModelSelector::chooseModel(const std::string& curModel,
                                                bool showOptions,
                                                bool showSkins)
 {
@@ -239,6 +330,21 @@ void ModelSelector::onSkinsOrModelsReloaded()
 
 void ModelSelector::onModelLoaded(const model::ModelNodePtr& modelNode)
 {
+    auto modelName = _modelPreview->getModel();
+    auto skinName = _modelPreview->getSkin();
+
+    if (modelName == _infoModel && skinName == _infoSkin)
+    {
+        return; // no change, don't clear everything
+    }
+
+    _infoModel = modelName;
+    _infoSkin = modelName;
+
+    _infoTable->Clear();
+    _materialsList->clear();
+    _relatedEntityStore->Clear();
+
 	if (!modelNode)
 	{
 		return;
@@ -247,25 +353,47 @@ void ModelSelector::onModelLoaded(const model::ModelNodePtr& modelNode)
 	// Update the text in the info table
 	const model::IModel& model = modelNode->getIModel();
 
-    auto modelName = _modelPreview->getModel();
 	_infoTable->Append(_("Model name"), modelName);
-	_infoTable->Append(_("Skin name"), _modelPreview->getSkin());
+	_infoTable->Append(_("Skin name"), skinName);
 	_infoTable->Append(_("Total vertices"), string::to_string(model.getVertexCount()));
 	_infoTable->Append(_("Total polys"), string::to_string(model.getPolyCount()));
 	_infoTable->Append(_("Material surfaces"), string::to_string(model.getSurfaceCount()));
 
-    if (modelName.find_last_of('/') == std::string::npos)
+    // Model name doesn't have a folder, this could be a modelDef
+    if (auto modelDef = GlobalEntityClassManager().findModel(modelName); modelDef)
     {
-        // Model name doesn't have a folder, this could be a modelDef
-        auto modelDef = GlobalEntityClassManager().findModel(modelName);
-
-        if (modelDef)
-        {
-            _infoTable->Append(_("Defined in"), modelDef->defFilename);
-        }
+        _infoTable->Append(_("Defined in"), modelDef->getBlockSyntax().fileInfo.fullPath());
     }
 
     _materialsList->updateFromModel(model);
+
+    wxDataViewItem matchingItem;
+
+    // Find related entity classes
+    GlobalEntityClassManager().forEachEntityClass([&] (const IEntityClassPtr& eclass)
+    {
+        auto modelKeyValue = eclass->getAttributeValue("model", true);
+
+        if (modelKeyValue == modelName)
+        {
+            auto row = _relatedEntityStore->AddItem();
+
+            auto eclassSkin = eclass->getAttributeValue("skin");
+
+            row[_relatedEntityColumns.eclassName] = eclass->getDeclName();
+            row[_relatedEntityColumns.skin] = eclassSkin;
+
+            if (!matchingItem.IsOk() && eclassSkin == skinName)
+            {
+                matchingItem = row.getItem();
+            }
+
+            row.SendItemAdded();
+        }
+    });
+
+    // Select the item that is matching the model/skin pair
+    _relatedEntityView->Select(matchingItem);
 }
 
 wxWindow* ModelSelector::setupTreeViewToolbar(wxWindow* parent)
@@ -296,7 +424,7 @@ void ModelSelector::setupTreeView(wxWindow* parent)
                     &ModelSelector::onTreeViewPopulationFinished, this);
 
     // Pack in tree view and its toolbar
-    parent->GetSizer()->Prepend(_treeView, 1, wxEXPAND);
+    parent->GetSizer()->Prepend(_treeView, 2, wxEXPAND);
     parent->GetSizer()->Prepend(setupTreeViewToolbar(parent), 0,
                                 wxEXPAND | wxALIGN_LEFT | wxBOTTOM | wxLEFT | wxRIGHT, 6);
     parent->GetSizer()->Layout();
@@ -321,24 +449,20 @@ void ModelSelector::Populate()
 
 void ModelSelector::onSelectionChanged(wxDataViewEvent& ev)
 {
-	showInfoForSelectedModel();
+    handleSelectionChange();
 }
 
 // Update the info table and model preview based on the current selection
-
-void ModelSelector::showInfoForSelectedModel()
+void ModelSelector::handleSelectionChange()
 {
-    // Prepare to populate the info table
-    _infoTable->Clear();
-
     // Get the model name, if this is blank we are looking at a directory,
     // so leave the table empty
-    std::string modelName = _treeView->GetSelectedModelPath();
+    auto modelName = _treeView->GetSelectedModelPath();
 
     if (modelName.empty()) return;
 
     // Get the skin if set
-    std::string skinName = _treeView->GetSelectedSkin();
+    auto skinName = _treeView->GetSelectedSkin();
 
     // Pass the model and skin to the preview widget
     _modelPreview->setModel(modelName);
@@ -347,12 +471,16 @@ void ModelSelector::showInfoForSelectedModel()
 
 void ModelSelector::onOK(wxCommandEvent& ev)
 {
-    // Remember the selected model then exit from the recursive main loop
-    _lastModel = _treeView->GetSelectedModelPath();
-    _lastSkin = _treeView->GetSelectedSkin();
-
 	_panedPosition.saveToPath(RKEY_SPLIT_POS);
 
+    _result.objectKind = Result::ObjectKind::Model;
+    _result.name = _treeView->GetSelectedModelPath();
+    _result.skin = _treeView->GetSelectedSkin();
+    _result.createClip = findNamedObject<wxCheckBox>(this, "ModelSelectorMonsterClipOption")->GetValue();
+
+    // Remember the last selected model
+    registry::setValue(RKEY_LAST_SELECTED_MODEL, _result.name);
+
 	EndModal(wxID_OK); // break main loop
 	Hide();
 }
