@@ -110,7 +110,7 @@ namespace
 
 void SurfaceInspector::ManipulatorRow::setValue(double v)
 {
-    value->SetValue(fmt::format("{}", v));
+    value->SetValue(fmt::format("{0:g}", v));
 }
 
 void SurfaceInspector::FitTextureWidgets::enable(bool enabled)
@@ -788,7 +788,7 @@ void SurfaceInspector::onShaderSelect(wxCommandEvent& ev)
     std::string previousShader = _shaderEntry->GetValue().ToStdString();
 
 	// Instantiate the modal dialog, will block execution
-	auto* chooser = new ShaderChooser(this, _shaderEntry);
+	auto* chooser = new ShaderChooser(this, ShaderSelector::TextureFilter::Regular, _shaderEntry);
 
     chooser->signal_shaderChanged().connect(
         sigc::mem_fun(this, &SurfaceInspector::emitShader)
