@@ -98,8 +98,6 @@ XYWnd::XYWnd(int id, wxWindow* parent) :
 
     _viewType = XY;
 
-    _contextMenu = false;
-
 	_wxGLWidget->SetCanFocus(false);
 	// Don't set a minimum size, to allow for cam window maximisation
 	//_wxGLWidget->SetMinClientSize(wxSize(XYWND_MINSIZE_X, XYWND_MINSIZE_Y));
@@ -173,7 +171,7 @@ void XYWnd::destroyXYView()
     _wxGLWidget = nullptr;
 }
 
-void XYWnd::setScale(float f) 
+void XYWnd::setScale(float f)
 {
     _scale = f;
     updateProjection();
@@ -224,13 +222,12 @@ int XYWnd::getDeviceHeight() const
 
 void XYWnd::captureStates()
 {
-    _highlightShaders.selectedShader = GlobalRenderSystem().capture("$XY_OVERLAY");
-    _highlightShaders.selectedShaderGroup = GlobalRenderSystem().capture("$XY_OVERLAY_GROUP");
-    _highlightShaders.mergeActionShaderAdd = GlobalRenderSystem().capture("$XY_MERGE_ACTION_ADD");
-    _highlightShaders.mergeActionShaderChange = GlobalRenderSystem().capture("$XY_MERGE_ACTION_CHANGE");
-    _highlightShaders.mergeActionShaderRemove = GlobalRenderSystem().capture("$XY_MERGE_ACTION_REMOVE");
-    _highlightShaders.mergeActionShaderConflict = GlobalRenderSystem().capture("$XY_MERGE_ACTION_CONFLICT");
-    _highlightShaders.nonMergeActionNodeShader = GlobalRenderSystem().capture("$XY_INACTIVE_NODE");
+    _highlightShaders.selectedShader = GlobalRenderSystem().capture(BuiltInShaderType::WireframeSelectionOverlay);
+    _highlightShaders.selectedShaderGroup = GlobalRenderSystem().capture(BuiltInShaderType::WireframeSelectionOverlayOfGroups);
+    _highlightShaders.mergeActionShaderAdd = GlobalRenderSystem().capture(BuiltInShaderType::OrthoMergeActionOverlayAdd);
+    _highlightShaders.mergeActionShaderChange = GlobalRenderSystem().capture(BuiltInShaderType::OrthoMergeActionOverlayChange);
+    _highlightShaders.mergeActionShaderRemove = GlobalRenderSystem().capture(BuiltInShaderType::OrthoMergeActionOverlayRemove);
+    _highlightShaders.mergeActionShaderConflict = GlobalRenderSystem().capture(BuiltInShaderType::OrthoMergeActionOverlayConflict);
 }
 
 void XYWnd::releaseStates()
@@ -241,7 +238,6 @@ void XYWnd::releaseStates()
     _highlightShaders.mergeActionShaderChange.reset();
     _highlightShaders.mergeActionShaderRemove.reset();
     _highlightShaders.mergeActionShaderConflict.reset();
-    _highlightShaders.nonMergeActionNodeShader.reset();
 }
 
 void XYWnd::ensureFont()
@@ -470,7 +466,7 @@ void XYWnd::onContextMenu()
 {
 	// Get the click point in 3D space
 	Vector3 point;
-	mouseToPoint(_contextMenu_x, _contextMenu_y, point);
+	mouseToPoint(_rightClickPos->initial.x(), _rightClickPos->initial.y(), point);
 
 	// Display the menu, passing the coordinates for creation
 	OrthoContextMenu::Instance().Show(_wxGLWidget, point);
@@ -502,27 +498,23 @@ EViewType XYWnd::getViewType() const {
 void XYWnd::handleGLMouseMotion(int x, int y, unsigned int state, bool isDelta)
 {
     // Context menu handling
-    if (state == wxutil::MouseButton::RIGHT) // Only RMB, nothing else
+    if (state == wxutil::MouseButton::RIGHT && _rightClickPos) // Only RMB, nothing else
     {
-        if (_contextMenu)
-        {
-            if (isDelta)
-            {
-                if (x != 0 || y != 0) // x,y are deltas
-                {
-                    // The user moved the pointer away from the point the RMB was pressed
-                    _contextMenu = false;
-                }
-            }
-            else
-            {
-                if (_contextMenu_x != x || _contextMenu_y != y)
-                {
-                    // The user moved the pointer away from the point the RMB was pressed
-                    _contextMenu = false;
-                }
-            }
-        }
+        if (isDelta)
+            _rightClickPos->current += Vector2i{x, y};
+        else
+            _rightClickPos->current = {x, y};
+
+        // If the user moved the pointer away from the point the RMB was pressed, this is a drag
+        // not a context menu event
+        static const std::string RKEY_THRESHOLD = std::string(RKEY_XYVIEW_ROOT)
+                                                + "/contextMenuThreshold";
+        static const int DIST_THRESHOLD_SQ = std::pow(
+            registry::getValue<int>(RKEY_THRESHOLD, 15), 2
+        );
+        auto distanceVec = _rightClickPos->current - _rightClickPos->initial;
+        if (distanceVec.getLengthSquared() > DIST_THRESHOLD_SQ)
+            _rightClickPos.reset();
     }
 
     _mousePosition = convertXYToWorld(x, y);
@@ -1374,6 +1366,8 @@ void XYWnd::draw()
     }
 
     {
+        GlobalRenderSystem().startFrame();
+
         // Construct the renderer and render the scene
         XYRenderer renderer(flagsMask, _highlightShaders);
 
@@ -1389,7 +1383,9 @@ void XYWnd::draw()
 		}
 
         // Second pass (GL calls)
-        renderer.render(_modelView, _projection);
+        renderer.render(_modelView, _projection, _view);
+
+        GlobalRenderSystem().endFrame();
     }
 
     glDepthMask(GL_FALSE);
@@ -1527,8 +1523,6 @@ void XYWnd::draw()
     // Reset the depth mask to its initial value (enabled)
     glDepthMask(GL_TRUE);
     debug::assertNoGlErrors();
-
-    glFinish();
 }
 
 void XYWnd::mouseToPoint(int x, int y, Vector3& point)
@@ -1584,7 +1578,7 @@ void XYWnd::zoomInOn(wxPoint cursor, int zoom)
     int dim2 = _viewType == XY ? 1 : 2;
 
     // worldPos = origin + devicePos * device2WorldScale
-    // devicePos and worldPos should remain constant. device2WorldScale is known before 
+    // devicePos and worldPos should remain constant. device2WorldScale is known before
     // and after zooming, so the origin delta can be calculated from what we have
     auto scaleAdjustment = (1 / _scale - 1 / newScale);
 
@@ -1676,9 +1670,8 @@ void XYWnd::onGLMouseButtonPress(wxMouseEvent& ev)
     if (ev.RightDown() && !ev.HasAnyModifiers())
     {
         // Remember the RMB coordinates for use in the mouseup event
-        _contextMenu = true;
-        _contextMenu_x = ev.GetX();
-        _contextMenu_y = ev.GetY();
+        Vector2i clickPos{ev.GetX(), ev.GetY()};
+        _rightClickPos = { clickPos, clickPos };
     }
 
     // Send the event to the mouse tool handler
@@ -1690,7 +1683,7 @@ void XYWnd::onGLMouseButtonPress(wxMouseEvent& ev)
 void XYWnd::onGLMouseButtonRelease(wxMouseEvent& ev)
 {
 	// Do the context menu handling first
-    if (ev.RightUp() && !ev.HasAnyModifiers() && _contextMenu)
+    if (ev.RightUp() && !ev.HasAnyModifiers() && _rightClickPos)
     {
         // The user just pressed and released the RMB in the same place
         onContextMenu();
