@@ -35,7 +35,9 @@ Face::Face(Brush& owner) :
     _owner(owner),
     _shader(texdef_name_default(), _owner.getBrushNode().getRenderSystem()),
     _undoStateSaver(nullptr),
-    _faceIsVisible(true)
+    _faceIsVisible(true),
+    _windingSurfaceSolid(m_winding, false),
+    _windingSurfaceWireframe(m_winding, true)
 {
     setupSurfaceShader();
 
@@ -58,7 +60,9 @@ Face::Face(
     _shader(shader, _owner.getBrushNode().getRenderSystem()),
     _texdef(projection),
     _undoStateSaver(nullptr),
-    _faceIsVisible(true)
+    _faceIsVisible(true),
+    _windingSurfaceSolid(m_winding, false),
+    _windingSurfaceWireframe(m_winding, true)
 {
     setupSurfaceShader();
     m_plane.initialiseFromPoints(p0, p1, p2);
@@ -70,7 +74,9 @@ Face::Face(Brush& owner, const Plane3& plane) :
     _owner(owner),
     _shader("", _owner.getBrushNode().getRenderSystem()),
     _undoStateSaver(nullptr),
-    _faceIsVisible(true)
+    _faceIsVisible(true),
+    _windingSurfaceSolid(m_winding, false),
+    _windingSurfaceWireframe(m_winding, true)
 {
     setupSurfaceShader();
     m_plane.setPlane(plane);
@@ -82,7 +88,9 @@ Face::Face(Brush& owner, const Plane3& plane, const Matrix3& textureProjection,
     _owner(owner),
     _shader(material, _owner.getBrushNode().getRenderSystem()),
     _undoStateSaver(nullptr),
-    _faceIsVisible(true)
+    _faceIsVisible(true),
+    _windingSurfaceSolid(m_winding, false),
+    _windingSurfaceWireframe(m_winding, true)
 {
     setupSurfaceShader();
     m_plane.setPlane(plane);
@@ -101,7 +109,9 @@ Face::Face(Brush& owner, const Face& other) :
     _shader(other._shader.getMaterialName(), _owner.getBrushNode().getRenderSystem()),
     _texdef(other.getProjection()),
     _undoStateSaver(nullptr),
-    _faceIsVisible(other._faceIsVisible)
+    _faceIsVisible(other._faceIsVisible),
+    _windingSurfaceSolid(m_winding, false),
+    _windingSurfaceWireframe(m_winding, true)
 {
     setupSurfaceShader();
     planepts_assign(m_move_planepts, other.m_move_planepts);
@@ -110,9 +120,12 @@ Face::Face(Brush& owner, const Face& other) :
 
 Face::~Face()
 {
-    _surfaceShaderRealised.disconnect();
+    _shader.setRealisedCallback({});
     _sigDestroyed.emit();
     _sigDestroyed.clear();
+
+    // Deallocate the winding surface
+    clearRenderables();
 }
 
 sigc::signal<void>& Face::signal_faceDestroyed()
@@ -122,8 +135,7 @@ sigc::signal<void>& Face::signal_faceDestroyed()
 
 void Face::setupSurfaceShader()
 {
-    _surfaceShaderRealised = _shader.signal_Realised().connect(
-        sigc::mem_fun(*this, &Face::realiseShader));
+    _shader.setRealisedCallback(std::bind(&Face::realiseShader, this));
 
     // If we're already in realised state, call realiseShader right away
     if (_shader.isRealised())
@@ -144,6 +156,8 @@ Brush& Face::getBrushInternal()
 
 void Face::planeChanged()
 {
+    updateRenderables();
+
     revertTransform();
     _owner.onFacePlaneChanged();
 }
@@ -159,6 +173,8 @@ void Face::connectUndoSystem(IUndoSystem& undoSystem)
 
     _shader.setInUse(true);
 
+    updateRenderables();
+
     _undoStateSaver = undoSystem.getStateSaver(*this);
 }
 
@@ -168,6 +184,9 @@ void Face::disconnectUndoSystem(IUndoSystem& undoSystem)
     _undoStateSaver = nullptr;
     undoSystem.releaseStateSaver(*this);
 
+    // Disconnect the renderable winding vertices from the scene
+    clearRenderables();
+
     _shader.setInUse(false);
 }
 
@@ -233,13 +252,6 @@ bool Face::intersectVolume(const VolumeTest& volume, const Matrix4& localToWorld
     }
 }
 
-void Face::renderWireframe(RenderableCollector& collector, const Matrix4& localToWorld,
-    const IRenderEntity& entity) const
-{
-    collector.addRenderable(*entity.getWireShader(), m_winding, localToWorld,
-                            nullptr, &entity);
-}
-
 void Face::setRenderSystem(const RenderSystemPtr& renderSystem)
 {
     _shader.setRenderSystem(renderSystem);
@@ -247,14 +259,9 @@ void Face::setRenderSystem(const RenderSystemPtr& renderSystem)
     // Update the visibility flag, we might have switched shaders
     const ShaderPtr& shader = _shader.getGLShader();
 
-    if (shader)
-    {
-        _faceIsVisible = shader->getMaterial()->isVisible();
-    }
-    else
-    {
-        _faceIsVisible = false; // no shader => not visible
-    }
+    _faceIsVisible = shader && shader->getMaterial()->isVisible();
+
+    clearRenderables();
 }
 
 void Face::transformTexDefLocked(const Matrix4& transform)
@@ -337,7 +344,23 @@ void Face::freezeTransform()
     updateWinding();
 }
 
-void Face::updateWinding() {
+void Face::clearRenderables()
+{
+    _windingSurfaceSolid.clear();
+    _windingSurfaceWireframe.clear();
+}
+
+void Face::updateRenderables()
+{
+    _windingSurfaceSolid.queueUpdate();
+    _windingSurfaceWireframe.queueUpdate();
+
+    _owner.onFaceNeedsRenderableUpdate();
+}
+
+void Face::updateWinding()
+{
+    updateRenderables();
     m_winding.updateNormals(m_plane.getPlane().normal());
 }
 
@@ -385,17 +408,9 @@ void Face::shaderChanged()
 
     // Update the visibility flag, but leave out the contributes() check
     const ShaderPtr& shader = getFaceShader().getGLShader();
+    _faceIsVisible = shader && shader->getMaterial()->isVisible();
 
-    if (shader)
-    {
-        _faceIsVisible = shader->getMaterial()->isVisible();
-    }
-    else
-    {
-        _faceIsVisible = false; // no shader => not visible
-    }
-
-    planeChanged();
+    planeChanged(); // updates renderables too
     SceneChangeNotify();
 }
 
@@ -432,6 +447,7 @@ void Face::texdefChanged()
 {
     revertTexdef();
     emitTextureCoordinates();
+    updateRenderables();
 
     // Fire the signal to update the Texture Tools
     signal_texdefChanged().emit();
@@ -681,6 +697,16 @@ Winding& Face::getWinding() {
     return m_winding;
 }
 
+render::RenderableWinding& Face::getWindingSurfaceSolid()
+{
+    return _windingSurfaceSolid;
+}
+
+render::RenderableWinding& Face::getWindingSurfaceWireframe()
+{
+    return _windingSurfaceWireframe;
+}
+
 const Plane3& Face::plane3() const
 {
     _owner.onFaceEvaluateTransform();
@@ -729,9 +755,29 @@ bool Face::isVisible() const
     return _faceIsVisible;
 }
 
+void Face::onBrushVisibilityChanged(bool visible)
+{
+    if (!visible)
+    {
+        // Disconnect our renderable when the owning brush goes invisible
+        clearRenderables();
+    }
+    else
+    {
+        // Update the vertex buffers next time we need to render
+        updateRenderables();
+    }
+}
+
 void Face::updateFaceVisibility()
 {
-    _faceIsVisible = contributes() && getFaceShader().getGLShader()->getMaterial()->isVisible();
+    auto newValue = contributes() && getFaceShader().getGLShader()->getMaterial()->isVisible();
+    
+    // Notify the owning brush if the value changes
+    if (newValue != _faceIsVisible)
+    {
+        _faceIsVisible = newValue;
+    }
 }
 
 sigc::signal<void>& Face::signal_texdefChanged()
