@@ -7,6 +7,8 @@
 #include "ilightnode.h"
 #include "registry/CachedKey.h"
 #include "scene/TransformedCopy.h"
+#include "math/Frustum.h"
+#include "editable.h"
 
 #include "dragplanes.h"
 #include "../VertexInstance.h"
@@ -265,6 +267,7 @@ public:
     Matrix4 getLightTextureTransformation() const override;
     Vector3 getLightOrigin() const override;
     bool isShadowCasting() const override;
+    bool isBlendLight() const override;
     const ShaderPtr& getShader() const override;
 	AABB lightAABB() const override;
 };
