@@ -40,7 +40,6 @@
 #include "map/algorithm/MapExporter.h"
 #include "model/export/ModelExporter.h"
 #include "model/export/ModelScalePreserver.h"
-#include "map/algorithm/Skins.h"
 #include "messages/ScopedLongRunningOperation.h"
 #include "messages/FileOverwriteConfirmation.h"
 #include "messages/FileSaveConfirmation.h"
@@ -999,17 +998,18 @@ void Map::registerCommands()
     GlobalCommandSystem().addCommand("SaveAutomaticBackup", std::bind(&Map::saveAutomaticMapBackup, this, std::placeholders::_1), { cmd::ARGTYPE_STRING });
     GlobalCommandSystem().addCommand("ExportMap", std::bind(&Map::exportMap, this, std::placeholders::_1));
     GlobalCommandSystem().addCommand("SaveSelected", Map::exportSelection);
-	GlobalCommandSystem().addCommand("ReloadSkins", map::algorithm::reloadSkins);
     GlobalCommandSystem().addCommand("FocusViews", std::bind(&Map::focusViewCmd, this, std::placeholders::_1), { cmd::ARGTYPE_VECTOR3, cmd::ARGTYPE_VECTOR3 });
     GlobalCommandSystem().addCommand("FocusCameraOnSelection", std::bind(&Map::focusCameraOnSelectionCmd, this, std::placeholders::_1));
-	GlobalCommandSystem().addCommand("ExportSelectedAsModel", map::algorithm::exportSelectedAsModelCmd,
-        { cmd::ARGTYPE_STRING,
-          cmd::ARGTYPE_STRING,
-          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL,
-          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL,
-          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL,
-          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL,
-          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL });
+    // ExportSelectedAsModel <Path> <ExportFormat> [<ExportOrigin>] [<OriginEntityName>] [<CustomOrigin>] [<SkipCaulk>] [<ReplaceSelectionWithModel>] [<ExportLightsAsObjects>]
+	GlobalCommandSystem().addCommand("ExportSelectedAsModel", algorithm::exportSelectedAsModelCmd,
+        { cmd::ARGTYPE_STRING, // path
+          cmd::ARGTYPE_STRING, // export format
+          cmd::ARGTYPE_STRING | cmd::ARGTYPE_OPTIONAL, // export origin type
+          cmd::ARGTYPE_STRING | cmd::ARGTYPE_OPTIONAL, // origin entity name
+          cmd::ARGTYPE_VECTOR3 | cmd::ARGTYPE_OPTIONAL, // custom origin
+          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL, // skip caulk
+          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL, // replace selection with model
+          cmd::ARGTYPE_INT | cmd::ARGTYPE_OPTIONAL }); // export lights as objects
 
     // Add undo commands
     GlobalCommandSystem().addCommand("Undo", std::bind(&Map::undoCmd, this, std::placeholders::_1));
@@ -1486,8 +1486,6 @@ const StringSet& Map::getDependencies() const
 
 void Map::initialiseModule(const IApplicationContext& ctx)
 {
-    rMessage() << getName() << "::initialiseModule called." << std::endl;
-
     // Register for the startup event
 	_mapPositionManager.reset(new MapPositionManager);
 
