@@ -401,14 +401,6 @@ OpenGLState& OpenGLShader::appendDepthFillPass()
     return _depthFillPass->state();
 }
 
-OpenGLState& OpenGLShader::appendInteractionPass()
-{
-    _interactionPass = std::make_shared<InteractionPass>(*this, _renderSystem);
-    _shaderPasses.push_back(_interactionPass);
-
-    return _interactionPass->state();
-}
-
 // Test if we can render in bump map mode
 bool OpenGLShader::canUseLightingMode() const
 {
@@ -452,12 +444,18 @@ void OpenGLShader::setGLTexturesFromTriplet(OpenGLState& pass,
     }
 }
 
-// Add an interaction layer
-void OpenGLShader::appendInteractionLayer(const DBSTriplet& triplet)
+OpenGLState& OpenGLShader::appendInteractionPass(std::vector<IShaderLayer::Ptr>& stages)
 {
+    // Store all stages in a single interaction pass
+    _interactionPass = std::make_shared<InteractionPass>(*this, _renderSystem, stages);
+    _shaderPasses.push_back(_interactionPass);
+
+    return _interactionPass->state();
+    
+
+#if 0
 	// Set layer vertex colour mode and alphatest parameters
     IShaderLayer::VertexColourMode vcolMode = IShaderLayer::VERTEX_COLOUR_NONE;
-    double alphaTest = -1;
 
     if (triplet.diffuse)
     {
@@ -465,20 +463,6 @@ void OpenGLShader::appendInteractionLayer(const DBSTriplet& triplet)
         alphaTest = triplet.diffuse->getAlphaTest();
     }
 
-    // Append a depthfill shader pass if requested
-    if (triplet.needDepthFill && triplet.diffuse)
-    {
-        // Create depth-buffer fill pass with alpha test
-        auto& zPass = appendDepthFillPass();
-
-        // Store the alpha test value
-        zPass.alphaThreshold = static_cast<GLfloat>(alphaTest);
-
-        // We need a diffuse stage to be able to performthe alpha test
-        zPass.stage0 = triplet.diffuse;
-        zPass.texture0 = getTextureOrInteractionDefault(triplet.diffuse)->getGLTexNum();
-    }
-
     // Add the DBS pass
     auto& dbsPass = appendInteractionPass();
 
@@ -499,7 +483,8 @@ void OpenGLShader::appendInteractionLayer(const DBSTriplet& triplet)
 	if (triplet.diffuse)
 	{
 		dbsPass.setColour(triplet.diffuse->getColour());
-	} 
+	}
+#endif
 }
 
 void OpenGLShader::applyAlphaTestToPass(OpenGLState& pass, double alphaTest)
@@ -515,18 +500,18 @@ void OpenGLShader::applyAlphaTestToPass(OpenGLState& pass, double alphaTest)
 // Construct lighting mode render passes
 void OpenGLShader::constructLightingPassesFromMaterial()
 {
-    // Build up and add shader passes for DBS triplets as they are found. A
-    // new triplet is found when (1) the same DBS layer type is seen twice, (2)
-    // we have at least one DBS layer then see a blend layer, or (3) we have at
-    // least one DBS layer then reach the end of the layers.
+    // Build up and add shader passes for DBS stages similar to the game code.
+    // DBS stages are first sorted and stored in the interaction pass where
+    // they will be evaluated and grouped to DBS triplets in every frame.
+    // All other layers are treated as independent blend layers.
 
-    DBSTriplet triplet;
-    const IShaderLayerVector& allLayers = _material->getAllLayers();
+    std::vector<IShaderLayer::Ptr> interactionLayers;
+    IShaderLayer::Ptr diffuseForDepthFillPass;
 
-    for (const auto& layer : allLayers)
+    _material->foreachLayer([&] (const IShaderLayer::Ptr& layer)
     {
         // Skip programmatically disabled layers
-        if (!layer->isEnabled()) continue;
+        if (!layer->isEnabled()) return true;
 
 		// Make sure we had at least one evaluation call to fill the material registers
 		layer->evaluateExpressions(0);
@@ -534,62 +519,59 @@ void OpenGLShader::constructLightingPassesFromMaterial()
         switch (layer->getType())
         {
         case IShaderLayer::DIFFUSE:
-            if (triplet.diffuse)
+            // Use the diffusemap with the highest opacity for the z-fill pass
+            if (!diffuseForDepthFillPass || 
+                (diffuseForDepthFillPass->getAlphaTest() != -1 && layer->getAlphaTest() == -1))
             {
-                appendInteractionLayer(triplet);
-                triplet.reset();
+                diffuseForDepthFillPass = layer;
             }
-            triplet.diffuse = layer;
+            interactionLayers.push_back(layer);
             break;
 
         case IShaderLayer::BUMP:
-            if (triplet.bump)
-            {
-                appendInteractionLayer(triplet);
-                triplet.reset();
-            }
-            triplet.bump = layer;
-            break;
-
         case IShaderLayer::SPECULAR:
-            if (triplet.specular)
-            {
-                appendInteractionLayer(triplet);
-                triplet.reset();
-            }
-            triplet.specular = layer;
+            interactionLayers.push_back(layer);
             break;
 
         case IShaderLayer::BLEND:
-            if (triplet.specular || triplet.bump || triplet.diffuse)
-            {
-                appendInteractionLayer(triplet);
-                triplet.reset();
-            }
-
             appendBlendLayer(layer);
         }
-    }
 
-    // Submit final pass if we reach the end
-    if (triplet.specular || triplet.bump || triplet.diffuse)
-	{
-		appendInteractionLayer(triplet);
-	}
+        return true;
+    });
+
+    // Sort interaction stages: bumps go first, then diffuses, speculars last
+    std::sort(interactionLayers.begin(), interactionLayers.end(), [](const IShaderLayer::Ptr& a, const IShaderLayer::Ptr& b)
+    {
+        // Use the enum value to sort stages
+        return static_cast<int>(a->getType()) < static_cast<int>(b->getType());
+    });
+
+    if (!interactionLayers.empty())
+    {
+        if (diffuseForDepthFillPass)
+        {
+            // Create depth-buffer fill pass, possibly with alpha test
+            auto& zPass = appendDepthFillPass();
+
+            zPass.stage0 = diffuseForDepthFillPass;
+            zPass.texture0 = getTextureOrInteractionDefault(diffuseForDepthFillPass)->getGLTexNum();
+            zPass.alphaThreshold = diffuseForDepthFillPass->getAlphaTest();
+        }
+
+        appendInteractionPass(interactionLayers);
+    }
 }
 
 void OpenGLShader::determineBlendModeForEditorPass(OpenGLState& pass)
 {
     bool hasDiffuseLayer = false;
+    IShaderLayer* firstLayer = nullptr;
 
     // Determine alphatest from first diffuse layer
-    const IShaderLayerVector allLayers = _material->getAllLayers();
-
-    for (IShaderLayerVector::const_iterator i = allLayers.begin();
-         i != allLayers.end();
-         ++i)
+    _material->foreachLayer([&](const IShaderLayer::Ptr& layer)
     {
-        const IShaderLayer::Ptr& layer = *i;
+        firstLayer = layer.get();
 
         if (layer->getType() == IShaderLayer::DIFFUSE)
         {
@@ -598,20 +580,22 @@ void OpenGLShader::determineBlendModeForEditorPass(OpenGLState& pass)
             if (layer->getAlphaTest() > 0)
             {
                 applyAlphaTestToPass(pass, layer->getAlphaTest());
-                break;
+                return false;
             }
         }
-    }
+
+        return true;
+    });
 
     // If this is a purely blend material (no DBS layers), set the editor blend
     // mode from the first blend layer.
 	// greebo: Hack to let "shader not found" textures be handled as diffusemaps
-    if (!hasDiffuseLayer && !allLayers.empty() && _material->getName() != "_default")
+    if (!hasDiffuseLayer && firstLayer != nullptr && _material->getName() != "_default")
     {
 		pass.setRenderFlag(RENDER_BLEND);
 		pass.setSortPosition(OpenGLState::SORT_TRANSLUCENT);
 
-		BlendFunc bf = allLayers[0]->getBlendFunc();
+		BlendFunc bf = firstLayer->getBlendFunc();
 		pass.m_blend_src = bf.src;
 		pass.m_blend_dst = bf.dest;
     }
@@ -626,6 +610,19 @@ void OpenGLShader::constructEditorPreviewPassFromMaterial()
     auto editorTex = _material->getEditorImage();
     previewPass.texture0 = editorTex ? editorTex->getGLTexNum() : 0;
 
+    // If there's a diffuse stage's, link it to this shader pass to inherit
+    // settings like scale and translate
+    _material->foreachLayer([&](const IShaderLayer::Ptr& layer)
+    {
+        if (layer->getType() == IShaderLayer::DIFFUSE)
+        {
+            previewPass.stage0 = layer;
+            return false;
+        }
+
+        return true;
+    });
+
     previewPass.setRenderFlag(RENDER_FILL);
     previewPass.setRenderFlag(RENDER_TEXTURE_2D);
     previewPass.setRenderFlag(RENDER_DEPTHTEST);
@@ -683,6 +680,13 @@ void OpenGLShader::appendBlendLayer(const IShaderLayer::Ptr& layer)
     // Set the texture
     state.texture0 = layerTex->getGLTexNum();
 
+    // BlendLights need to load the fall off image into texture unit 1
+    if (_material->isBlendLight())
+    {
+        state.texture1 = _material->lightFalloffImage()->getGLTexNum();
+        state.setRenderFlag(RENDER_CULLFACE);
+    }
+
     // Get the blend function
     BlendFunc blendFunc = layer->getBlendFunc();
     state.m_blend_src = blendFunc.src;
@@ -709,6 +713,12 @@ void OpenGLShader::appendBlendLayer(const IShaderLayer::Ptr& layer)
         state.setRenderFlag(RENDER_TEXTURE_CUBEMAP);
         state.clearRenderFlag(RENDER_TEXTURE_2D);
     }
+    else if (_material && _material->isBlendLight())
+    {
+        state.glProgram = _renderSystem.getGLProgramFactory().getBuiltInProgram(ShaderProgram::BlendLight);
+        state.setRenderFlag(RENDER_TEXTURE_2D);
+        state.setRenderFlag(RENDER_PROGRAM);
+    }
     else
     {
         state.glProgram = _renderSystem.getGLProgramFactory().getBuiltInProgram(ShaderProgram::RegularStage);
@@ -718,6 +728,7 @@ void OpenGLShader::appendBlendLayer(const IShaderLayer::Ptr& layer)
 
     // Colour modulation
     state.setColour(layer->getColour());
+    state.setVertexColourMode(layer->getVertexColourMode());
 
 	// Sort position
     if (_material->getSortRequest() >= Material::SORT_DECAL)
