@@ -1005,15 +1005,21 @@ void registerCommands()
 
 	GlobalCommandSystem().addCommand("ExportSelectedAsCollisionModel", createCMFromSelection, { cmd::ARGTYPE_STRING });
 
-	GlobalCommandSystem().addCommand("CreateDecalsForFaces", createDecalsForSelectedFaces);
+    GlobalCommandSystem().addWithCheck(
+        "CreateDecalsForFaces", [](const cmd::ArgumentList&) { createDecalsForSelectedFaces(); },
+        [] { return !FaceInstance::Selection().empty(); }
+    );
 
-	GlobalCommandSystem().addCommand("Copy", clipboard::copy);
+    GlobalCommandSystem().addCommand("Copy", clipboard::copy);
 	GlobalCommandSystem().addCommand("Cut", clipboard::cut);
 	GlobalCommandSystem().addCommand("Paste", clipboard::paste);
 	GlobalCommandSystem().addCommand("PasteToCamera", clipboard::pasteToCamera);
 
-	GlobalCommandSystem().addCommand("ConnectSelection", connectSelectedEntities);
-    GlobalCommandSystem().addCommand("BindSelection", bindEntities);
+    GlobalCommandSystem().addWithCheck("ConnectSelection", cmd::noArgs(connectSelectedEntities),
+                                       [] { return selection::pred::haveEntitiesExact(2); });
+    GlobalCommandSystem().addWithCheck("BindSelection", cmd::noArgs(bindEntities),
+                                       [] { return selection::pred::haveEntitiesExact(2); });
+
     GlobalCommandSystem().addCommand("PlacePlayerStart", placePlayerStart, { cmd::ARGTYPE_VECTOR3 });
 	GlobalCommandSystem().addCommand("SetEntityKeyValue", setEntityKeyValueOnSelection, { cmd::ARGTYPE_STRING, cmd::ARGTYPE_STRING });
     GlobalCommandSystem().addCommand("CreateCurveNURBS", createCurveNURBS);
